require("AnAL")
require("bounding_box")


function love.load()

    love.physics.setMeter(64) --the height of a meter our worlds will be 64px
     world = love.physics.newWorld(0, 9.81*64, true) --create a world for the bodies to exist in with horizontal gravity of 0 and vertical gravity of 9.81

    objects = {} -- table to hold all our physical objects
  
    --let's create the ground
    objects.ground = {}
    objects.ground.body = love.physics.newBody(world, 750, 600) --remember, the shape (the rectangle we create next) anchors to the body from its center, so we have to move it to (650/2, 650-50/2)
    objects.ground.shape = love.physics.newRectangleShape(800, 50) --make a rectangle with a width of 650 and a height of 50
    objects.ground.fixture = love.physics.newFixture(objects.ground.body, objects.ground.shape); --attach shape to body
  
    objects.char = {}
    objects.char.image = love.graphics.newImage("textures/hero2.png")
    objects.char.body = love.physics.newBody(world, 300, 400, "dynamic")
    objects.char.shape = love.physics.newRectangleShape(objects.char.image:getWidth(), objects.char.image:getHeight())
    objects.char.fixture = love.physics.newFixture(objects.char.body, objects.char.shape)
    objects.char.body:setFixedRotation(true)
    objects.char.body:setX( 50 )
    objects.char.body:setY( 500 )
    objects.char.y_velocity = 0
    anim = newAnimation(objects.char.image, 100, 100, 1, 1)

    objects.crate = {}
    objects.crate.image = love.graphics.newImage("textures/crate.png")
    objects.crate.body = love.physics.newBody(world, 500, 544, "static")
    objects.crate.shape = love.physics.newRectangleShape(0, 0, objects.crate.image:getWidth(), objects.crate.image:getHeight())
    objects.crate.fixture = love.physics.newFixture(objects.crate.body, objects.crate.shape, 5)
    objects.crate.body:setFixedRotation(true)
    objects.crate.fixture:setRestitution(0.4)
    objects.crate.fixture:setUserData("Block")

-------------------------------------------
    bullets = {}

    bulletSpeed = 250
    
    char_direction = "right"

    gravity = 400
    jump_height = 300

    bullet_image_right = love.graphics.newImage("textures/bullet_right.png")
    bullet_image_left = love.graphics.newImage("textures/bullet_left.png")

    winW, winH = love.graphics.getWidth(), love.graphics.getHeight()

    love.keyboard.setKeyRepeat(0.01, 0.05)
    love.graphics.setBackgroundColor(198, 198, 198, 255)

end

function love.draw()

    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.draw(objects.crate.image, objects.crate.body:getX(), objects.crate.body:getY())

    love.graphics.setColor(255, 255, 255, 255)
    if char_direction == "right" then
        anim:draw(objects.char.body:getX(), objects.char.body:getY() )
    elseif char_direction == "left" then
        anim:draw(objects.char.body:getX(), objects.char.body:getY(), objects.char.body:getAngle(),  -1, 1, 100, 0)
    end
    
    love.graphics.setColor(128, 128, 128)
    for i,v in ipairs(bullets) do
        love.graphics.draw(v.image, v.x, v.y)
    end
end

function love.update(dt)
    world:update(dt)
    anim:update(dt)   

    walking_speed, jumping_speed = objects.char.body:getLinearVelocity()


    for i = #bullets, 1, -1 do
        if bullets[i].direction == "right" then
            bullets[i].x = bullets[i].x + (bulletSpeed * dt)
        elseif bullets[i].direction == "left" then
            bullets[i].x = bullets[i].x - (bulletSpeed * dt)
        end

        if bullets[i].x > winW or bullets[i].x < 0 then
            table.remove(bullets, i)
        end
    end

    --if objects.char.y_velocity ~= 0 then -- we're probably jumping
    x, y =  objects.char.body:getLinearVelocity()
    if y ~= 0 then
        objects.char.body:setY( objects.char.body:getY() - objects.char.y_velocity * dt )
        --objects.char.y_velocity = objects.char.y_velocity - gravity * dt 
        y =  y - gravity * dt 
        objects.char.body:setLinearVelocity( x, y )
        if objects.char.body:getY() > 500 then -- we hit the ground again
            --objects.char.y_velocity = 0
            objects.char.body:setLinearVelocity( x, 0 )
            objects.char.body:setY( 500 )  
        end
    end
end

function love.keypressed( key )

    if key == "a" then
        objects.char.body:applyLinearImpulse( -10, 0 )
        char_direction = "left"
    elseif key == "d" then
        objects.char.body:applyLinearImpulse( 10, 0 )
        char_direction = "right"
    end

    if key == "w" then
        --if objects.char.y_velocity == 0 then -- we're probably on the ground, let's jump
        x, y =  objects.char.body:getLinearVelocity()
        if y == 0 then
            objects.char.body:setLinearVelocity( walking_speed, -200 )
            --objects.char.y_velocity = jump_height
        end
    end

end


function love.mousepressed(x, y, button)
    if button == "l" then
        local startX = objects.char.body:getX() + 50
        local startY = objects.char.body:getY() + 65
        
        if char_direction == "right" then
            bimage = bullet_image_right
        elseif char_direction == "left" then
            bimage = bullet_image_left
        end

        table.insert(bullets, {x = startX, y = startY, direction = char_direction, image = bimage})
    end
end

